#pragma once
typedef unsigned __int8 uint8_t;
typedef unsigned __int16 uint16_t;
typedef __int32 int32_t;
typedef unsigned __int32 uint32_t;
typedef int Err;

// Block message format:
// 3 byte marker = 'FTH'
// 1 byte command
// 1 byte block length high byte
// 1 byte block length low byte
// 1 byte header-crc high byte
// 1 byte header-crc low byte
// x byte user-data (depends on block type)
// rest: not yet defined

// replies with RQ_OK, RQ_TRANSFERERR or RQ_FAILED (=file exists)

enum{
 RQ_OK=27,		// binary 0011011      // block OK
 RQ_FAILED=73,		// binary 1001001      // block OK, operation failed
 RQ_TRANSFERERR=85,	// binary 1010101      // block transfer error
 RQ_READY=119,		// binary 1110111      // signal "transfer ready"
 RQ_CRCERR=99,		// binary 1100011      // block has bad CRC
 RQ_WAIT=60,		// binary 0111100      // tell the client to wait an extended time - for operations taking more than 5 seconds
 RQ_LOCALABORT=0,	// binary 0000000      // user aborted operation - never sent across line! Use RQ_FAILED instead
 RQ_LOCALTIMEOUT=1,	// binary 0000001      // Send or receive call timed out -> retry after 5 seconds delay
 RQ_F='F'};		// binary 1000110      // Start of a new block (other direction) -> abort operation!

// commands are uppercase, data blocks lowercase
enum{
 CMD_ABORT	='1',
 CMD_RECVERR	='2',
 CMD_CRCERR	='3',
 CMD_TIMEOUT	='4',
 CMD_LIST	='L',
 CMD_GET	='G',
 CMD_SEND	='S',
 CMD_DELETE	='D',
 CMD_RENFR	='F',
 CMD_RENTO	='T',
 CMD_COPYTO	='O',
 CMD_MKDIR	='M',
 CMD_CONNECT	='C',
 CMD_SETATTR	='A',
 CMD_SETTIME	='I',
 CMD_RESTART	='R',
 CMD_GETSIZE	='Q',
 CMD_OVERWRITE	='W',
 CMD_SIZE	='s',
 CMD_BODY	='b',
 CMD_EOF	='e',
};

#define maxblockpart 8192

enum{err_ok,err_abort,err_recv,err_send,err_mem,err_file};

const uint32_t maxblocksizeusb=63*1024;		// as large as possible
const uint32_t maxblocksizeserial=1*1024;	// XModem-1k block size
const int maxblocksize=maxblocksizeusb;

//blockio.h
int SendNextBlock(char request,char* databuf,uint32_t datalen,uint8_t sequencenr);
char ReceiveNextBlock(char* recvbody,uint32_t maxbodysize,uint32_t* datareceived,uint8_t* sequencenr);
int SendNextBlockRetry10(char request,char* databuf,int datalen,uint8_t sequencenr);

//lowlvlio.cpp
bool ConnectToPort(int portnum,int portspeed);
void ClosePorts();
void FreePortResources();
void ClearBuffers(DWORD what=PURGE_RXCLEAR|PURGE_TXCLEAR);
bool Connected();
Err TimedSendBytes(const char* sendbuf,DWORD numbytes,int32_t to_ms);
int TimedSendChar(char byte,int timeout_msec);
Err TimedReceiveBytes(char* recvbuf,DWORD&numbytes,int32_t to_ms);
int TimedReceiveChar(char* ch,int32_t timeout_msec);

//sertrans.cpp
bool SerialConnect(int portno,int portspeed);
int SerialSendFile(HANDLE fh,char* filename,char* block,DWORD resumeat, BOOL sizeonly);
int SerialReceiveFile(HANDLE fh,char* fileName,char* block,DWORD restartat);
void SerialClose();
int SerialSendChar(char byte);
//was:utils.cpp
bool ConvertIsoDateToDateTime(const char*s,FILETIME&ft);
bool CreateIsoDateString(const FILETIME&ft,char*s); //yyyymmddhhmmss
WORD Crc16CalcBlock(char* p, int len, WORD crc16);
